# logging utlities

import logging

# logs on shell
def slogger (name):
    log = logging.getLogger (name)
    log.setLevel (logging.INFO)
    formatter = logging.Formatter('[%(asctime)s %(levelname)s] %(message)s', "%d-%m-%Y %H:%M:%S")
    handler = logging.StreamHandler()
    handler.setFormatter(formatter)
    log.addHandler(handler)
    return log


# logs on file
def logger (name, basepath):
    log = logging.getLogger(name)
    log.setLevel (logging.DEBUG)
    formatter = logging.Formatter('[%(asctime)s %(levelname)s] %(message)s', "%d-%m-%Y %H:%M:%S")

    # info
    hfile_info = logging.FileHandler(basepath+"info.log")
    hfile_info.setFormatter(formatter)
    hfile_info.setLevel(logging.INFO)
    log.addHandler(hfile_info)

    # error
    hfile_error = logging.FileHandler(basepath+"error.log")
    hfile_error.setFormatter(formatter)
    hfile_error.setLevel(logging.ERROR)
    log.addHandler(hfile_error)
    
    # debug
    hfile_debug = logging.FileHandler(basepath+"debug.log")
    hfile_debug.setFormatter(formatter)
    hfile_debug.setLevel(logging.DEBUG)
    log.addHandler(hfile_debug)
    
    
    # return
    return log





import re, os
from file import read_csv
from copy import deepcopy
base_path = os.path.abspath(os.path.dirname(__file__))

def parseSymbol2Entrez(filepath):
    data = read_csv(filepath)
    localdict = {}
    for row in data[1:]:
        row = row[0].split(" ")
        if row[1].startswith('"') and row[1].endswith('"'):
            row[1] = row[1][1:-1]
        localdict[row[1]] = row[0]
    return localdict

entrezLookup = parseSymbol2Entrez(base_path + '/../data/symbol2entrez.txt')
#formats file by modifying file
def formatFeatureFile(filepath):
    locallist = []
    f = open(filepath, 'r')
    for row in f:
        row = re.findall(r"[\w']+", row)
        for item in row:
            if item in entrezLookup:
                locallist.append(entrezLookup[item])
            elif item in entrezLookup.values():
                locallist.append(item)
            else:
                print item
    f.close()
    return locallist

#formats list by modifying it in place
def formatFeatureList(featureList):
    localList = []
    for row in featureList:
        if isinstance(row, basestring):
            row = re.findall(r"[\w']+", row)
            for item in row:
                if item in entrezLookup:
                    localList.append(entrezLookup[item])
                elif item in entrezLookup.values():
                    localList.append(item)
                else:
                    print item
        elif isinstance(row, int):
            if row in entrezLookup.values():
                localList.append(row)
            else:
                print row
    return localList

def formatFeatureString(featureString):
    featureList = []
    for item in re.findall(r"[\w']+", featureString):
        if item in entrezLookup:
            featureList.append(entrezLookup[item])
        elif item in entrezLookup.values():
            featureList.append(item)
        else:
            print item
    return featureList

#takes genelists as an array, modifies array
def formatGeneLists(geneLists):
    for index, genelist in enumerate(geneLists):
        if genelist:
            if isinstance(genelist, basestring):
                if os.path.isfile(genelist):
                    geneLists[index] = formatFeatureFile(genelist)
                else:
                    geneLists[index] = formatFeatureString(genelist)
            elif isinstance(geneList, list):
                geneLists[index] = formatFeatureList(genelist)
            else:
                print genelist
    geneLists = filter(lambda x: x, map(lambda x: map(int, x) if x else None, geneLists))
    return geneLists


from lib.file import write_JSON, read_file, dictionaryListToCSV
from lib.errorHandling import printException
import time, re, os, threading, getopt
from copy import deepcopy
from operator import itemgetter
from multiprocessing import cpu_count, Value as mpVal
from ctypes import c_double

from CGEA import __file__ as package_path
package_path = os.path.dirname(package_path)

def get_CGEA_data():
    import  urllib, zipfile
    from distutils.sysconfig import get_python_lib
    input = raw_input("Performing first time setup - is it ok to download CGEA data (135.4 MB)? y/n\n")
    while not (input == 'y' or input == 'n'):
        input = raw_input("Invalid input. Permission to download CGEA data (135.4 MB)? y/n\n")
    if input == 'y':
        urllib.urlretrieve("https://bitbucket.org/MaxTomlinson/cgea/downloads/data.zip", 
                           package_path+"/data.zip")
        zipfile.ZipFile(package_path+"/data.zip").extractall(package_path+"/")
        os.remove(package_path+"/data.zip")
        print "Loaded All CGEA Data"
        return True
    else:
        raise RuntimeError('CGEA data not available.')
    
if not os.path.isfile(package_path+'/data/cmap_1309_entrez.json'):
    data_loaded = get_CGEA_data()
    
from lib.featureBasedCGEA import featureBasedCGEA
from lib.fishersBasedCGEA import fishersBasedCGEA
    
class geneListCGEA():
    def __init__(self, upgenes, downgenes, *args, **kwargs):
        self.myCGEA = initCGEA(upgenes = upgenes, downgenes = downgenes, *args, **kwargs)
        self.status = self.myCGEA.status
        self.results = self.myCGEA.results
        self.filepaths = self.myCGEA.filepaths
        
    def run(self):
        self.myCGEA.run()

class drugListCGEA():
    def __init__(self, druglist, *args, **kwargs):
        self.myCGEA = initCGEA(druglist = druglist, *args, **kwargs)
        self.status = self.myCGEA.status
        self.results = self.myCGEA.results
        self.filepaths = self.myCGEA.filepaths
        
    def run(self):
        self.myCGEA.run()
    
def initCGEA(async = False, *args, **kwargs):
    if async:
        myCGEA = CGEA_async(*args, **kwargs)
    else:
        myCGEA = CGEA(*args, **kwargs)
    return myCGEA
    
class CGEA_async(threading.Thread):
    def __init__(self, *args, **kwargs):
        threading.Thread.__init__(self)
        self.daemon = True
        self.myCGEA = CGEA(*args, **kwargs)
        self.status = self.myCGEA.status
        self.results = self.myCGEA.results
        self.filepaths = self.myCGEA.filepaths
        
    def run(self):
        self.myCGEA.run()


class CGEA():
    def __init__(self, upgenes = None, downgenes = None, druglist = None, saveAs = "json", sort = 'up', outputDir = None, prefix = 'CGEAResults', data = {}, numWorkers = cpu_count(), verbose = False):
        self.upgenes = upgenes
        self.downgenes = downgenes
        self.druglist = druglist
        self.status = ['PENDING','Initialized', 0.0]
        self.saveAs = saveAs
        self.outputDir = outputDir
        self.sort = sort
        self.prefix = str(prefix)
        self.data = data
        self.filepaths = []
        self.results = {}
        self.numWorkers = numWorkers
        self.verbose = verbose
        self.saveFunction = {'json': lambda: self.writeSingleSheet('.json', write_JSON),
                             #'json': lambda: self.writeMultipleSheets('.json', write_JSON),
                             #'csv':lambda: self.writeSingleSheet('.csv', lambda x,y: write_csv(x,y, delimiter = ',')),
                             #'tsv':lambda: self.writeSingleSheet('.tsv', lambda x,y: write_csv(x,y, delimiter = '\t')),
                             'csv':lambda: self.writeMultipleSheets('.csv', lambda x,y: dictionaryListToCSV(x,y, delimiter = ',', encode = 'utf8')),
                             'tsv':lambda: self.writeMultipleSheets('.tsv', lambda x,y: dictionaryListToCSV(x,y, delimiter = '\t', encode = 'utf8'))
                             }
    
    def writeResults(self):
        self.saveFunction[self.saveAs]()
    
    def writeSingleSheet(self, suffix, writeFxn):
        filepath = self.outputDir+self.prefix+suffix
        print filepath
        writeFxn(filepath, self.results)
        self.filepaths.append(filepath)
    
    def writeMultipleSheets(self, suffix, writeFxn):
        filepaths = []
        for name, result in self.results.items():
            if len(self.prefix) > 0: filepath = self.outputDir+re.sub('\s+', '_', '_'.join([self.prefix,name])+suffix)
            else: filepath = self.outputDir+re.sub('\s+', '_', name+suffix)
            self.filepaths.append(filepath)
            writeFxn(filepath, result)
#         
#     def writeEnrichmentsToJSONs(self):
#         filepaths = []
#         for name, result in self.results.items():
#             if len(self.prefix) > 0: filepath = self.outputDir+re.sub('\s+', '_', '_'.join([self.prefix,name])+'.json')
#             else: filepath = self.outputDir+re.sub('\s+', '_', name+'.json')
#             self.filepaths.append(filepath)
#             write_JSON(self.outputDir+re.sub('\s+', '_', ' '.join(self.prefix,name)+'.json'), result)
# 
#     def writeEnrichmentsToJSON(self):
#         filepath = self.outputDir+self.prefix+'.json'
#         write_JSON(filepath, self.results)
#         self.filepaths.append(filepath)
    
    def run(self):
        self.status[0] = 'RUNNING'
        if self.verbose:
            print 'Started CGEA'
        try:
            if self.upgenes == None and self.downgenes == None and self.druglist == None:
                raise RuntimeError("No input given to CGEA.")
            elif not (self.upgenes == None and self.downgenes == None):
                self.results.update(featureBasedCGEA(self.upgenes, self.downgenes, self.saveAs, self.outputDir, self.sort, 
                                                     self.data, self.status, numWorkers = self.numWorkers, verbose = self.verbose))
            else:
                self.results.update(fishersBasedCGEA(self.druglist, self.outputDir, self.sort, self.data, 
                                                     self.status, numWorkers = self.numWorkers, verbose = self.verbose))
            if self.saveAs and os.path.isdir(self.outputDir):
                curTime = time.time()
                self.writeResults()
                if self.verbose:
                    print 'Wrote Results'
            self.status[0] = 'SUCCESS'
            self.status[1] = 'Finished'
            self.status[2] = 1.0
            if self.verbose:
                print 'Finished CGEA'
        except Exception as e:
            printException()
            print e
            self.status[0] = 'ERROR'
            self.status[1] = e
            self.status[2] = 1.0
        return self.results

def testGeneListCGEA(**kwargs):
    upgenes = package_path+"/data/sampledata/up_genes.txt"
    downgenes = package_path+"/data/sampledata/down_genes.txt"
    outputDir = package_path+"/data/sampledata/sampleout/"
    curTime = time.time()
    geneListCGEA(upgenes = upgenes, downgenes = downgenes, prefix = 'sampleGeneListOutput', outputDir = outputDir, verbose = True, **kwargs).run()
    print 'Total time: ' + str(time.time()-curTime)

def testDrugListCGEA(**kwargs):
    druglist = package_path+"/data/sampledata/drug_list.txt"
    outputDir = package_path+"/data/sampledata/sampleout/"
    curTime = time.time()
    drugListCGEA(druglist = druglist, prefix = 'sampleDrugListOutput', outputDir = outputDir, verbose = True, **kwargs).run()
    print 'Total time: ' + str(time.time()-curTime)


from CGEAdata import CGEAdata
import multiprocessing as mp
from file import read_JSON, write_JSON, read_file
import os, time
from copy import deepcopy
from operator import itemgetter
from statistics import compoundSetHypergeometricEnrichment
import itertools, re
from drugListProcessing import drugListProcessing
from CGEA import __file__ as package_path
package_path = os.path.dirname(package_path)

def fishersBasedCGEA(drugList, outputDir, sortDir, data, status, alpha = .05, numWorkers = mp.cpu_count(), verbose = True):
    
    status[1] = 'Started'
    status[2] = 0.01
    curTime = time.time()
    data = CGEAdata(data = data)
    status[1] = 'Loading cmap data'
    if verbose:
        print status[1]
    cmap = data.get('cmapDrugMetaDataWithOffsides')
    dlp = drugListProcessing(cmap)
    print "Loaded cmap data - took " + str(time.time()-curTime) + ' seconds'
    status[1] = 'Loaded cmap data'
    if verbose:
        print status[1]
    status[2] = 0.03
    
    drugList = list(itertools.chain(*map(lambda x: re.findall(r"[\w']+", x),
                                         read_file(drugList))))
    #print 'Drug list: '+str(drugList)
    
    hitNames = set(map(lambda x: x[0].lower(), 
                       filter(lambda x: x[1],
                              dlp.inCMAP(drugList))
                       ))
    cmapNames = set(map(lambda x: x['Name'].lower(), cmap))
    annotatedCompounds = dlp.annotateCMAP(hitNames)
    #print 'Hits: ' + str(hitNames)
    #print 'Hit[0]: ' + str(hits[0])
    #print 'Misses: ' + str(misses)
    
    
    compoundSetResults = {}
    
    
    minCompoundSetSizeThreshold = 5
    compoundListsFilepath = package_path+'/data/CompoundLists_SizeThreshold-'+str(minCompoundSetSizeThreshold)+'.json'
    if not os.path.isfile(compoundListsFilepath):
        startTime = time.time()
        unifiedCompoundLevelSets = data.get("unifiedCompoundLevelSets")
        enrichmentData = data.getEnrichmentData()
        compoundLists = unifiedCompoundLevelSets.items()+enrichmentData.items()
        compoundListsNames = map(itemgetter(0), compoundLists)
        compoundLists = list(itertools.chain(*map(lambda x: map(lambda y: (x[0],
                                                                           y[0],
                                                                           map(lambda z: z.lower(),
                                                                               filter(lambda z: dlp.inCMAP(z),
                                                                                      y[1]))), 
                                                                x[1].items()), 
                                                  compoundLists)))
        compoundLists = filter(lambda x: len(x[2])>minCompoundSetSizeThreshold, compoundLists)
        write_JSON(compoundListsFilepath, compoundLists)
        #print 'Writing compoundList took: ' + str(time.time()-startTime) + ' seconds.'
    else:
        startTime = time.time()
        compoundLists = read_JSON(compoundListsFilepath)
        #print 'Loading compoundList took: ' + str(time.time()-startTime) + ' seconds.'
    
    compoundSets = map(lambda x: set(x[2]), compoundLists)
    
    compoundSetScores = compoundSetHypergeometricEnrichment(hitNames, compoundSets, len(cmapNames), numWorkers)
    summaryResults = map(lambda x: {'Compound Set':x[1][0], 
                                    'Name':x[1][1], 
                                    'P-Value':compoundSetScores[x[0]][0],
                                    'Adjusted P-Value':compoundSetScores[x[0]][1],
                                    'Overlap Set': set.intersection(hitNames, compoundSets[x[0]])
                                    }, enumerate(compoundLists))
    for x in range(len(summaryResults)):
        if len(summaryResults[x]['Overlap Set']) == 0:
            summaryResults[x]['Overlap Set'] = 'NA'
        else:
            summaryResults[x]['Overlap Set'] = ';'.join(sorted(summaryResults[x]['Overlap Set']))
    #print sorted(summaryResults, key = itemgetter('Adjusted P-Value'))[:10]
    results = {'Annotated Compounds':annotatedCompounds, 'Combined Enrichment Results':summaryResults}
    return results



from file import read_JSON
import os
from sklearn.externals import joblib
dir = os.path.dirname(__file__)

class CGEAdata():
    #Default file locations for data
    data = {"cmap":"data/cmap_1309_entrez.json", 
            "cmapDrugMetaData" : "data/cmapDrugMetaDataOfficial.json", 
            "cmapDrugMetaDataWithOffsides" : "data/cmapDrugMetaDataWithOffsides.json",
            "cmap1309map" : "data/drugPRLidMap.txt",
            "drugbank_carriers_cmap" : "data/annotations/drugbank/carriers_cmap.json", 
            "drugbank_carriers_all" : "data/annotations/drugbank/carriers_all.json",
            "drugbank_targets_cmap" : "data/annotations/drugbank/targets_cmap.json", 
            "drugbank_targets_all" : "data/annotations/drugbank/targets_all.json",
            "drugbank_enzymes_cmap" : "data/annotations/drugbank/enzymes_cmap.json",
            "drugbank_enzymes_all" : "data/annotations/drugbank/enzymes_all.json",
            "drugbank_transporters_cmap" : "data/annotations/drugbank/transporters_cmap.json",
            "drugbank_transporters_all" : "data/annotations/drugbank/transporters_all.json",
            "SEA_alltargets_cmap" : "data/annotations/SEA_alltargets_cmap.json",
            "SEA_alltargets_cmap_MDDR" : "data/annotations/SEA_alltargets_cmap_MDDR.json",
            "ATC_3" : "data/annotations/ATC_3.json",
            "ATC_4" : "data/annotations/ATC_4.json",
            "drugSets_CODIM" : "data/annotations/drugSets_CODIM.json",
            "molFragmentsSize" : "data/annotations/molFragmentsSize.json",
            "sideFx" : "data/annotations/sideFx.json",
            "unifiedCompoundLevelSets" : "data/annotations/unifiedCompoundLevelSets.json",
            "offsides" : "data/annotations/offsides.json",
            "mixmdl" : "data/mixmdl/mixmdl_1309.pkl",
            "nullscores" : "data/nullScores_1309.json"
            }
    
    def __init__(self, data = None):
        for key, value in self.data.items():
            if not os.path.isfile(value):
                self.data[key] = os.path.join(dir, '../'+value)
        self.enrichmentData = {'Known Target Enrichment':self.data['drugbank_targets_cmap'],
                               'Known Enzyme Enrichment':self.data['drugbank_enzymes_cmap'],
                               'Known Transporter Enrichment':self.data['drugbank_transporters_cmap'],
                               'Known Carrier Enrichment':self.data['drugbank_carriers_cmap'],
                               'Known and Predicted Enrichment':self.data['SEA_alltargets_cmap'],
                               'ATC3 Enrichments':self.data['ATC_3'],
                               'ATC4 Enrichments':self.data['ATC_4'],
                               'DITM Enrichments':self.data['drugSets_CODIM'],
                               'Fragment Enrichments':self.data['molFragmentsSize'],
                               'Side Effects - Enrichments':self.data['sideFx'],
                               'Offsides - Enrichments':self.data['offsides']
                               }
        if data:
            try:
                for key, value in files.items():
                    if not key in data:
                        self.data[key] = value
            except TypeError:
                print "Warning: Invalid data input. Must input as a python dictionary"
    
    def get(self, identifier):
        if os.path.isfile(self.data[identifier]):
            return read_JSON(self.data[identifier])
        else:
            return False
    
    def getEnrichmentData(self):
        output = {}
        for key, value in self.enrichmentData.items():
            output[key] = read_JSON(value)
        return output
    
    def getPath(self, identifier):
        return self.data[identifier]
    
    def getModel(self, identifier):
        if os.path.isfile(self.data[identifier]):
            return joblib.load(self.data[identifier]) 
        else:
            return False


#Generates scores, pvals, background models
import random, math, time
from copy import deepcopy
import numpy as np
from sklearn import mixture, preprocessing
from sklearn.externals import joblib
from statsmodels.sandbox.stats.multicomp import multipletests
from scipy.stats import hypergeom
from operator import itemgetter
from SimpleMapReduce import SimpleMapReduce
from errorHandling import printException
import itertools
import multiprocessing as mp   

def multiMap(mapFunc, inputs, numWorkers, chunksize, pool = None):
    if not pool:
        pool = mp.Pool(numWorkers)
    #print inputs
    #print mapFunc(inputs[0])
    #print chunksize
    if numWorkers > 1:
        results = pool.map(mapFunc, inputs, chunksize=chunksize)
    else:
        results = map(mapFunc, inputs)
#     if results[0] == None: 
#         print inputs
#         print mapFunc(inputs[0])
#         print mapFunc(inputs[1])
#         print chunksize
#         print numWorkers
    pool.close()
    pool.join()
    return results

#geneLists, cmap
def scoreRankedLists(featureLists, rankedLists, status = None, numWorkers = mp.cpu_count(), pool = None):
    nameIndex = {}
    index = {}
    #startTime = time.time()
    scoreData = multiMap(scoreRankedList(featureLists, rankedLists, status),
                         rankedLists.keys(),
                         numWorkers,
                         chunksize=int(math.ceil(float(len(rankedLists.keys()))/numWorkers)),
                         pool = pool)
    for count, data in enumerate(scoreData):
        name = data[0]
        index[count] = name
        nameIndex[name] = count
    #print scoreData
    scores = map(lambda x: [x[1]], scoreData)
    #print scores
    return [nameIndex, scores, index]

class scoreRankedList():
    def __init__(self, featureLists, rankedLists, status):
        self.featureLists = featureLists
        self.rankedLists = rankedLists
        #self.status = status
        self.countLists = len(rankedLists.keys())
    
    def __call__(self, name):
#         if self.status:
#             self.status[2].value += 1.0/(self.countLists)*3.3/10.0
        cmapList = self.rankedLists[name]
        return [name, generateScore(self.featureLists, cmapList)[0]]

class genPval():
    def __init__(self, weights):
        self.weights = weights
    def __call__(self, x):
        return sum(map(lambda y: pow(10,math.log(x[1][y]/self.weights[y],10)+x[0]),range(0,len(self.weights))))

def generatePVals(backgroundModel, scores, setSizes = None, backgroundScores = None, scaler = None, alpha = .05, FDR = None, numWorkers = mp.cpu_count(), pool = None):
    if not isinstance(backgroundModel, dict):
        if not FDR: FDR = calcFDR(backgroundModel, backgroundScores, alpha = alpha)
        if scaler: scores = scaler.transform(scores)
        #print scores
        pvals = map(genPval(backgroundModel.weights_), zip(*backgroundModel.score_samples(scores)))
        scores = map(itemgetter(0), scores)
    else:
        pvals = []
        FDRs = []
        for index, score in enumerate(scores):
            setSize = setSizes[index]
            if not setSize in backgroundModel:
                setSize = max(backgroundModel.keys())
            model = backgroundModel[setSize]['model']
            FDRs.append(backgroundModel[setSize]['FDR'])
            scaler = backgroundModel[setSize]['scaler']
            pvals.append(map(genPval(model.weights_), 
                             zip(*model.score_samples(scaler.transform(np.array([score]).reshape(1, -1))))
                             )[0] #only one value
                         )
        FDR = np.mean(FDRs)
    adjpvals = multipletests(pvals, method='fdr_bh', alpha = FDR)[1]
    return [list(pvals), list(adjpvals), list(scores)]

def calcFDR(backgroundModel, backgroundScores, alpha = .05):
    backgroundPvals = map(genPval(backgroundModel.weights_), zip(*backgroundModel.score_samples(backgroundScores)))
    FDR = sorted(backgroundPvals)[int(math.floor(len(backgroundPvals)*alpha))]
    return FDR
    

#query takes the featureLists and queries it against randomized rankedLists.
#randomSubset takes a random subset of the featureList of size 'sizeOfSubset', and queries it against a randomized rankedList
backgroundModelScoringMethods = {'query':lambda x,y,z,w,s,nw,p: generateBackgroundScores(x,y,z,status=s,numWorkers = nw, pool = p),
                                 'randomSubset':lambda x,y,z,w,s,nw,p:
                                                        reduce(lambda a,b: a+b, #joins list of background scores
                                                            map(lambda placeholder: 
                                                                    generateBackgroundScores(random.sample(x,w),
                                                                                             sorted(y, key=lambda k: random.random()),
                                                                                             int(math.ceil(math.sqrt(z))),
                                                                                             numWorkers = nw,
                                                                                             pool = p),
                                                                range(0, int(math.ceil(math.sqrt(z)))) 
                                                            )#gens list of background scores based on subset of rankedList
                                                        )
                          }
n_components = {'query': 3,
                'randomSubset': 2
                }
def generateBackgroundModel(featureLists, rankedList, numPermutations = 1000, sizeOfSubset = None, method = 'query', status = None, numWorkers = mp.cpu_count(), pool = None):
    scores = backgroundModelScoringMethods[method](featureLists, rankedList, numPermutations, sizeOfSubset, status, numWorkers, pool)
    
    scaler = preprocessing.StandardScaler().fit(scores)
    scores = scaler.transform(scores)
    model = mixture.GMM(n_components=n_components[method], covariance_type = 'diag', min_covar = .001, tol= 1e-05, n_iter = 10000, n_init = 200) #tol is convergence threshold
    model.fit(scores)
    return [model,list(scores), scaler]

def generateBackgroundScores(featureLists, rankedList, numPermutations, status = None, numWorkers = mp.cpu_count(), pool = None):
    
    generateBackgroundScoreFunction = generateBackgroundScore(featureLists, rankedList, numPermutations, status)
    #mapper = SimpleMapReduce(generateBackgroundScoreFunction, lambda x: list(itertools.chain.from_iterable(x)), num_workers = numWorkers)
    #startTime = time.time()
    scores = multiMap(generateBackgroundScoreFunction,
                      range(numPermutations), 
                      numWorkers,
                      chunksize=int(math.ceil(float(numPermutations)/numWorkers)),
                      pool = pool)
    #scores = mapper(range(numPermutations),chunksize=int(math.ceil(numPermutations/numWorkers)))
    return scores

class generateBackgroundScore():
    def __init__(self, featureLists, rankedList, numPermutations, status):
        self.featureLists = featureLists
        self.rankedList = rankedList
        self.numPermutations = numPermutations
        #self.status = status
    
    def __call__(self, runNumber):
#         print runNumber
#         if self.status:
#             print self.status.value
#             self.status.value += 1.0/(self.numPermutations)*3.0/10.0
        
        if isinstance(self.rankedList,dict):
            unrankedList = sorted(self.rankedList[random.choice(self.rankedList.keys())], key=lambda k: random.random())
        elif isinstance(self.rankedList, list):
            unrankedList = sorted(self.rankedList, key=lambda k: random.random())
        else:
            #print self.rankedList
            unrankedList = sorted(self.rankedList, key=lambda k: random.random())
        #if (i+1)%100 == 0:print str(i+1) + ' permutations.'
        return [generateScore(self.featureLists, unrankedList)[0]]
        
    

#For a list of genes, generates a score for each drug in cmap. 19 lines.
def generateScore(featureLists, rankedList):
    if not isinstance(featureLists[0], list): featureLists = [featureLists]
    unrankedSet = set(rankedList)
    lenRankedList = len(rankedList)
    #for each up/down in featureList
    peakValues = []
    peakValueIndices = []
    hitPositions = []
    for featureList in featureLists:
        featureSet = set(featureList)
        #Remove values in the featureList not in rankedList
        featureSet = set.intersection(featureSet, unrankedSet)
        numHits = len(featureSet)
        if numHits > 0:
            numMisses = lenRankedList - numHits
            
            #print "numGenes is: "+ str(len(featureList)) +", numRankedGenes is: "+ str(len(cmapSet))+" numHits is: " + str(numHits) + ", numMisses is: " + str(numMisses)
            #hits is a cumulative probability distribution for the likelihood that a gene will appear in the list.
            #ie, if rankedList was [1,2,3,4,5,6,7,8,9], and featureList was [2,5,8,9], 
            #hits would be [0,.25,.25,.25,.5,.5,.5,.75,1]
            #misses would be [.2,.2,.4,.6,.6,.8,1,1,1]
            #difference would be [.2,.05,.15,.35,.1,.3,.5,.25,0]
            hits = map(lambda x: int(x in featureSet), rankedList)
            hitPositions.append(deepcopy(hits))
            hits = np.cumsum(hits)
            hits = map(lambda x: float(x)/numHits, hits)
            
            misses = np.cumsum(map(lambda x: int(not x in featureSet), rankedList))
            misses = map(lambda x: float(x)/numMisses, misses)
            
            difference = map(lambda x: abs(hits[x] - misses[x]), range(lenRankedList))
            
            #find the value that represent the greatest difference between ranked list and gene set.
            peakValue = max(difference)
            peakIndex = difference.index(peakValue)
            peakValueIndices.append(peakIndex)
            peakValues.append(hits[peakIndex]-misses[peakIndex])
    if len(peakValues) == 1:
        score = peakValues[0]
        peakValueIndices = peakValueIndices[0]
        hitPositions = hitPositions[0]
    elif len(featureLists) ==2:
        score = (peakValues[0] - peakValues[1])/2 #assume featureList[1] is down signature
    else:
        raise ValueError
    return [score, peakValueIndices, hitPositions]

def compoundSetGSEA(rankedDrugs, mixmdls, compoundSets, alpha = .05, numWorkers = mp.cpu_count(), pool = None):
    scores = []
    peakPositions = []
    hitPositionLists = []
    setSizes = []
    #print compoundSets
    #print scoreCompoundSet(compoundSets, rankedDrugs)(0)
    scoreData = multiMap(scoreCompoundSet(compoundSets, rankedDrugs),
                         range(len(compoundSets)), 
                         min(numWorkers,len(compoundSets)),
                         chunksize=int(math.ceil(float(len(compoundSets))/numWorkers)),
                         pool = pool)
    [scores, peakPositions, hitPositionLists, setSizes] = zip(*scoreData)
    [pvals, adjpvals, scores] = generatePVals(mixmdls, scores, setSizes = setSizes, alpha = alpha)
    output = []
    for index, [compoundName, _] in enumerate(compoundSets):
        output.append({'Name': compoundName,
                       'P-Value': pvals[index], 
                       'Adjusted P-Value':adjpvals[index], 
                       'Score':scores[index], 
                       'Peak Position':peakPositions[index], 
                       'Hit Positions':hitPositionLists[index]})
    return output
#looking for DC_PATHWAY_UP and [u'flunisolide', u'beta-escin', u'medrysone', u'DL-thiorphan', u'sitosterol', u'phenoxybenzamine']
class scoreCompoundSet():
    def __init__(self, compoundSets, rankedDrugs):
        self.compoundSets = compoundSets
        self.rankedDrugs = rankedDrugs
        
    def __call__(self, index):
        [compoundName, compoundSet] = self.compoundSets[index]
        [score, peakPosition, hitPositions] = generateScore(compoundSet, self.rankedDrugs)
        hitPositions = map(itemgetter(0), filter(lambda x: x[1], enumerate(hitPositions)))
        hitPositions = ';'.join(map(lambda x: self.rankedDrugs[x] + ':' + str(x), hitPositions))
        setSize = len(compoundSet)
        return [score, peakPosition, hitPositions, setSize]

def compoundSetHypergeometricEnrichment(querySet, compoundSets, universeSize, numWorkers, pool = None):
    compoundSetPvals = multiMap(setHypergeometricEnrichmentClass(querySet, universeSize),
                         compoundSets, 
                         min(numWorkers,len(compoundSets)),
                         chunksize=int(math.ceil(float(len(compoundSets))/numWorkers)),
                         pool = pool)
    compoundSetAdjPvals = multipletests(compoundSetPvals, method='fdr_bh', alpha = .05)[1]
    return zip(*[compoundSetPvals, compoundSetAdjPvals])

class setHypergeometricEnrichmentClass():
    def __init__(self, querySet, universeSize):
        self.querySet, self.universeSize = querySet, universeSize
        
    def __call__(self, popB):
        return setHypergeometricEnrichmentScore(self.querySet, popB, self.universeSize)

def setHypergeometricEnrichmentScore(popA, popB, popSize):
    if not (isinstance(popA, set) and isinstance(popB, set)):
        print 'inputs should be sets in compoundSetHypergeometricEnrichment'
        return False
    
    popASize = len(popA)
    popBSize = len(popB)
    popAB = set.intersection(popA, popB)
    popABSize = len(popAB)
    #sanity check
    if not (len(set.union(popA,popB)) + popABSize == popASize + popBSize ):
        print 'popA: '+str(popA)
        print 'popB: '+str(popB)
        print 'popAB: '+str(popAB)
        print 'something is rotten in the state of setHypergeometricEnrichment function.'
        
    return hypergeom.sf(*[popABSize-1, popSize, popASize, popBSize])#P(X >= popABSize)

def makeMixmdl(featureList, filepath, maxSetSize = 50, numIterations = 100, alpha = .05, setSizes = None):
    models = {}
    if not setSizes:
        setSizes = set(range(1,maxSetSize+1))
    for i in setSizes:
        print 'Iteration#'+str(i)
        [model, scores, scaler] = generateBackgroundModel(featureList, featureList, numPermutations = numIterations, sizeOfSubset = i, method = 'randomSubset')
        models[i] = {'model':model,
                     'scaler':scaler,
                     'FDR': calcFDR(model, scores, alpha = alpha)}
        print np.histogram(scores)
        print "Background weights:" +str(model.weights_)
        print "Background means:" +str(map(itemgetter(0),model.means_))
        print "Background covars:" +str(map(itemgetter(0),model.covars_))
    joblib.dump(models, filepath) 


from CGEA import drugListCGEA, testGeneListCGEA, testDrugListCGEA

if __name__ == '__main__' :
    testGeneListCGEA(numWorkers = 1, async = False, saveAs = 'csv', sort = 'abs')
    testDrugListCGEA(numWorkers = 1, async = False, saveAs = 'tsv')


from setuptools import setup


setup(name='CGEA',
      version='0.909',
      description='Chemo Genomic Enrichment Analysis',
      url='https://bitbucket.org/MaxTomlinson/cgea',
      author='Max Tomlinson / Ben Readhead',
      author_email='max.tomlinson@mssm.edu',
      license='MIT',
      packages=['CGEA'],
      install_requires=[
          'jsonpickle',
          'numpy',
          'scikit-learn',
          'scipy',
          'statsmodels',
          'pandas'
      ],
      include_package_data=True,
      zip_safe=False)





from lib.featureBasedCGEA import CGEAdata
from lib.statistics import makeMixmdl

if __name__ == '__main__':  
    print 'Loading Data'
    data = CGEAdata()
    cmap = data.get('cmap')
    unifiedCompoundLevelSets = data.get("unifiedCompoundLevelSets")
    mixmdls = data.getModel('mixmdl')
    setSizes = reduce(lambda x,y: set.union(x,set(map(len,y.values()))), 
                      unifiedCompoundLevelSets.values(), 
                      set())
    print 'Generating mixmdls'
    makeMixmdl(cmap.keys(), data.getPath('mixmdl'), setSizes = setSizes, numIterations = 100000)
